package org.switchyard.quickstarts.helloworld.topic.notifier;

import static org.hamcrest.Matchers.*;
import static org.junit.Assert.*;

import java.io.File;
import java.io.IOException;

import javax.jms.Destination;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.MessageProducer;
import javax.jms.Session;
import javax.jms.TextMessage;

import org.apache.commons.io.FileUtils;
import org.hornetq.jms.client.HornetQTopic;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.switchyard.component.camel.model.RouteScanner;
import org.switchyard.component.test.mixins.hornetq.HornetQMixIn;
import org.switchyard.test.SwitchYardRunner;
import org.switchyard.test.SwitchYardTestCaseConfig;

@RunWith(SwitchYardRunner.class)
@SwitchYardTestCaseConfig(
        config = SwitchYardTestCaseConfig.SWITCHYARD_XML,
        mixins = HornetQMixIn.class,
        scanners = RouteScanner.class)
public class HelloWorldTopicNotifierTest {
    
    private static final String QUEUE_NAME = "quickstart_helloworld_topic_notifier_Request";
    private static final String TEST_MESSAGE = "Hello World";
    private static final String OUTPUT_FILE = "target/output/results.log";
    private static final String TOPIC_NAME = "helloworldtopic";
    
    private HornetQMixIn hornetQMixIn;
    
    @Before
    public void setUp() {
        new File(OUTPUT_FILE).delete();
    }
    
    @Test
    public void sendJMS() throws Exception {
        //send(HornetQMixIn.getJMSQueue(QUEUE_NAME), TEST_MESSAGE);
        send(new HornetQTopic(QUEUE_NAME), TEST_MESSAGE);
        assertOutputFile();
        receiveAndAssert(new HornetQTopic(TOPIC_NAME), TEST_MESSAGE);
        //receiveAndAssert(HornetQMixIn.getJMSQueue(TOPIC_NAME), TEST_MESSAGE);
    }
    private void send(Destination destination, String text) throws Exception {
        Session session = null;
        try {
            session = hornetQMixIn.createJMSSession();
            MessageProducer producer = session.createProducer(destination);
            Message message = hornetQMixIn.createJMSMessage(text);
            producer.send(message);
            Thread.sleep(500);
        } finally {
            HornetQMixIn.closeJMSSession(session);
        }
    }
    
    private void assertOutputFile() throws IOException {
        File outputFile = new File(OUTPUT_FILE);
        assertThat(outputFile.exists(), is(true));
        assertThat(FileUtils.readFileToString(outputFile), is(equalTo(TEST_MESSAGE)));
    }
    
    private void receiveAndAssert(Destination destination, String expectedText) throws Exception {
        Session session = null;
        try {
            session = hornetQMixIn.createJMSSession();
            MessageConsumer consumer = session.createConsumer(destination);
            Message message = consumer.receive(3000);
            assertThat(message, is(instanceOf(TextMessage.class)));
            assertThat(((TextMessage) message).getText(), is(equalTo(expectedText)));
        } finally {
            HornetQMixIn.closeJMSSession(session);
        }
    }
    
}
